local DrawSprites = class("DrawSprites", System)

function DrawSprites:initialize(map)
  System.initialize(self)
  self.map = map

  self.spriteLayer = self.map.layers["Sprites"]
  self.spriteLayer.sprites = {}

  function self.spriteLayer:draw()
    if self.sprites ~= nil then
      for _, sprite in pairs(self.sprites) do
        if sprite.blink ~= true or sprite.blink == true
           and sprite.blinkingTime > math.floor(sprite.blinkingTime)+sprite.blinkingFrequency then
          -- if we have a sprite image, we use it, else we juste draw a rectangle
          --local imagePath = entity:get("Sprite").imagePath
          --if (imagePath == "") then
            love.graphics.setColor(sprite.color[1], sprite.color[2], sprite.color[3], sprite.color[4])
            love.graphics.rectangle(sprite.style, sprite.x,
            sprite.y, sprite.w, sprite.h)
            if sprite.name ~= nil then
              love.graphics.setColor(0, 0, 0, 1)
              love.graphics.print(sprite.name, sprite.x-2, sprite.y)
            end
            love.graphics.setColor(1, 1, 1, 1)
          --end
        end
      end
    end
  end
end

function DrawSprites:draw(dt)
  local n = 1
  for _, entity in pairs(self.targets) do
    if (entity:get("Movable").type == 'player') then
      local sprite = entity:get('Sprite')
      -- If our player isn't blinkg, we reset the blinking timer
      if sprite.blink == true and sprite.blinkingTime < sprite.maxBlinkingTime then
        -- If our player is blinking, we increment the blinking timer
        local dt = love.timer.getDelta()
        sprite.blinkingTime = sprite.blinkingTime+dt
      elseif sprite.blinkingTime >= sprite.maxBlinkingTime then
        sprite.blinkingTime = 0
        sprite.blink = false
      end
      self.map.layers["Sprites"].sprites.player = {
        x = entity:get("Collidable").x,
        y = entity:get("Collidable").y,
        w = entity:get("Collidable").width,
        h = entity:get("Collidable").height,
        color = sprite.color,
        style = sprite.style,
        type = entity:get('Movable').type,
        blink = sprite.blink,
        blinkingTime = sprite.blinkingTime,
        blinkingFrequency = sprite.blinkingFrequency
      }
    else
      self.map.layers["Sprites"].sprites['enemy' .. n] = {
        x = entity:get("Collidable").x,
        y = entity:get("Collidable").y,
        w = entity:get("Collidable").width,
        h = entity:get("Collidable").height,
        color = entity:get('Sprite').color,
        style = entity:get('Sprite').style,
        type = entity:get('Movable').type,
        name = entity:get('Movable').name
      }
      n = n+1
    end
  end
end

function DrawSprites:requires()
    return {"Sprite"}
end

return DrawSprites
