local MoveCharacters = class("MoveCharacters", System)

function MoveCharacters:initialize(map, world)
  System.initialize(self)
  self.map = map
  self.world = world
end

function MoveCharacters:update(dt)
  --local sprites = self.map.layers["Sprites"].sprites
  for _, entity in pairs(self.targets) do
    local playableComp = entity:get("Playable")
    local movableComp = entity:get("Movable")
    local collidableComp = entity:get("Collidable")
    local positionComp = entity:get("Position")

    --[[local sprite = nil
    for _, sp in pairs(self.map.layers["Sprites"].sprites) do
			if sp.type == movableComp.type then
        sprite = sp
      end
    end
    if sprite ~= nil then]]
      local fX = collidableComp.x
      local fY = collidableComp.y
      local checkX = 1
      local checkY = 1

      if movableComp.fDirection == "u" then
        fY = collidableComp.y-1
        checkY = checkY-1
      end
      if movableComp.fDirection == "d" then
        fY = collidableComp.y+1
        checkY = checkY+1
      end
      if movableComp.fDirection == "l" then
        fX = collidableComp.x-1
        checkX = checkX-1
      end
      if movableComp.fDirection == "r" then
        fX = collidableComp.x+1
        checkX = checkX+1
      end

      if movableComp.fDirection ~= '' then
        -- Getting the tile coordinates of the tile the player is on
        local tX, tY = self.map:convertPixelToTile(collidableComp.x,
        collidableComp.y)

        -- Convert them to the coordinates of the "next" tile, according to
        -- the future direction (fDirection) of the player
        local tileCheckX = math.floor(tX)+checkX
        local tileCheckY = math.floor(tY)+checkY

        -- Fetching the properties of this tile
        local properties = self.map:getTileProperties('Calque 1', tileCheckX,
        tileCheckY)

        -- If "collidable" property is absent (== nil) or the element is a
        -- one-way element, then we can change direction
        if properties['collidable'] == nil or properties['direction'] == movableComp.fDirection then
          -- if the player is very close to the limit of the tile, we set its
          -- position to that limit
          if movableComp.fDirection ~= movableComp.direction then
            local limitX, limitY = self.map:convertTileToPixel(math.floor(tX), math.floor(tY))
            if (movableComp.direction == 'd' or movableComp.direction == 'u')
            and (movableComp.fDirection == 'r' or movableComp.fDirection == 'l') then
              if collidableComp.y > limitY and
              collidableComp.y-limitY < movableComp.speed * dt then
                collidableComp.y = limitY
                --positionComp.y = limitY
                movableComp.direction = ''
              end
            elseif (movableComp.direction == 'r' or movableComp.direction == 'l')
            and (movableComp.fDirection == 'd' or movableComp.fDirection == 'u') then
              if collidableComp.x > limitX and
              collidableComp.x-limitX < movableComp.speed * dt then
                collidableComp.x = limitX
                --positionComp.x = limitX
                movableComp.direction = ''
              end
            end
          end
        end

        local futureX, futureY, cols, len = self.world:check(collidableComp, fX, fY,
          function(item, other) return self:unidirectional_Col(item, other) end)
        if (movableComp.fDirection == "u" or movableComp.fDirection == "d") and fY == futureY or
        (movableComp.fDirection == "l" or movableComp.fDirection == "r") and fX == futureX then
          movableComp.direction = movableComp.fDirection
          movableComp.fDirection = ""
        end
      end

      local target_x = collidableComp.x
      local target_y = collidableComp.y

      if movableComp.direction == "u" then
        target_y = collidableComp.y - movableComp.speed * dt
      elseif movableComp.direction == "d" then
        target_y = collidableComp.y + movableComp.speed * dt
      elseif movableComp.direction == "l" then
        target_x = collidableComp.x - movableComp.speed * dt
      elseif movableComp.direction == "r" then
        target_x = collidableComp.x + movableComp.speed * dt
      end

      local conf_x, conf_y = world:move(collidableComp, target_x, target_y,
        function(item, other) return self:unidirectional_Col(item, other) end)

      collidableComp.prevX = collidableComp.x
      collidableComp.prevY = collidableComp.y
      collidableComp.x = conf_x
      collidableComp.y = conf_y
      --positionComp.x = conf_x
      --positionComp.y = conf_y
      self.world:update(collidableComp, conf_x, conf_y)
      --sprite.x = collidableComp.x
      --sprite.y = collidableComp.y
    --end
  end
end

function MoveCharacters:requires()
  return {"Movable", "Collidable"}
end

function MoveCharacters:unidirectional_Col(item, other)
  if item.entity ~= nil and other.entity ~= nil then
    if item.entity:get('Enemy') ~= nil and other.entity:get('Enemy') ~= nil then
      return 'cross'
    else
      local player = nil
      local enemy = nil
      if item.entity:get('Playable') ~= nil then
        player = item.entity
        enemy = other.entity
      else
        player = other.entity
        enemy = item.entity
      end
      if player:get('Sprite').blink == false then
        local iX, iY = self.map:convertPixelToTile(item.x, item.y)
        local oX, oY = self.map:convertPixelToTile(other.x, other.y)
        --[[debug:addTextToShow('item', item.x .. ':' .. item.y, true)
        debug:addTextToShow('itemTile', iX .. ':' .. iY, true)
        debug:addTextToShow('other', other.x .. ':' .. other.y, true)
        debug:addTextToShow('otherTile', oX .. ':' .. oY, true)]]
        pause = true
        player:get('Playable').lives = player:get('Playable').lives-1
        player:get('Sprite').blink = true
      else -- If our player is blinking, then he is not affected by enemies
        return 'cross'
      end
    end
  elseif other.properties ~= nil and other.properties['direction'] ~= nil then
    local entityDirection = item.entity:get("Movable").direction
    local entityFDirection = item.entity:get("Movable").fDirection

    if entityDirection == other.properties['direction'] or
       entityFDirection == other.properties['direction'] then
      return 'cross'
    end
  end

  return 'slide'
end

return MoveCharacters
