local MoveEnemies = class("MoveEnemies", System)

function MoveEnemies:initialize(map)
  System.initialize(self)
  self.directions = {'u', 'r', 'l', 'd'}
  self.time = 0
  self.timeBeforeChange = 2
  self.map = map
  self.safeZoneExit = nil
  n = 0

  -- Seeking enemie's safeZone exit
  for _, item in pairs(map.bump_collidables) do
    if item.properties ~= nil and item.properties['direction'] ~= nil then
      --debug:addTextToShow('okok', item.x .. ':' .. item.y, true)
      self.safeZoneExit = {
        x = item.x,
        y = item.y,
        w = item.width,
        h = item.height,
        direction = item.properties['direction']
      }
      break
    end
  end
end

function MoveEnemies:update(dt)
  for _, enemy in pairs(self.targets) do
    local movableComp = enemy:get("Movable")
    local collidableComp = enemy:get("Collidable")
    -- We want enemies to go out of the safe zone !
    if enemy:get('Enemy').safeZone == true then
      if collidableComp.y ~= self.safeZoneExit.y+self.safeZoneExit.h then
        movableComp.fDirection = 'u'
      elseif collidableComp.x ~= self.safeZoneExit.x then
        if collidableComp.x < self.safeZoneExit.x then
          movableComp.fDirection = 'r'
        else
          movableComp.fDirection = 'l'
        end
      end
      if movableComp.direction == 'r' or movableComp.direction == 'l' then
        movableComp.fDirection = 'u'
      end

      if collidableComp.y < self.safeZoneExit.y then
        enemy:get('Enemy').safeZone = false
      end
    else
      self.time = self.time + dt
      math.randomseed(dt)
      -- Our poor enemy doesn't know where to go
      if collidableComp.x == collidableComp.prevX and collidableComp.y == collidableComp.prevY then
        -- we make a new directions tabble without the one where he is blocked
        local newDirections = {}
        for _, direction in pairs(self.directions) do
          if direction ~= movableComp.direction then
            table.insert(newDirections, direction)
          end
        end

        movableComp.fDirection = self.directions[math.random(4)]
      end

      if self.time > self.timeBeforeChange then
        movableComp.fDirection = self.directions[math.random(4)]

        self.time = 0
      end
    end
  end
end

function MoveEnemies:requires()
  return {"Movable", "Collidable", "Enemy"}
end

return MoveEnemies
