sti = require 'libs.sti'
bump = require 'libs.bump.bump'

Hoo = require 'libs.HooECS'
Hoo.initialize({globals = true})

-- Requiring components
require 'ecs.components.position'
require 'ecs.components.sprite'
require 'ecs.components.map'
require 'ecs.components.score'
require 'ecs.components.size'
require 'ecs.components.collidable'
require 'ecs.components.movable'
require 'ecs.components.playable'
require 'ecs.components.enemy'

-- Requiring entities

-- Requiring systems
DrawMap = require 'ecs.systems.drawMap'
DrawSprites = require 'ecs.systems.drawSprites'
DrawHud = require 'ecs.systems.drawHud'
MovePlayer = require 'ecs.systems.movePlayer'
MoveEnemies = require 'ecs.systems.moveEnemies'
MoveCharacters = require 'ecs.systems.moveCharacters'

Level1 = class("Level1", State)

function Level1:initialize()
  self.tilesWidth = 16
  self.tilesHeight = 16
  --require 'ecs.entities.player'
end

function Level1:load()
  --local position = HooECS.Component.load("Position")

  self.engine = Engine()
  self.eventmanager = EventManager()
  --self.score = 0

  level = Entity()
  level:add(Score())
  level:add(Map("mac-pan-01.lua", {"bump"}))
  self.engine:addEntity(level)
  local player = nil
  local enemies = {}

  local map = level:get("Map").stiData
  -- Get player spawn object
  -- Create player entity
  local enemyN = 1
  for k, object in pairs(map.objects) do
    if object.type == "player" then
      player = Entity()
      player:add(Position(object.x, object.y))
      player:add(Size(self.tilesWidth, self.tilesHeight))
      player:add(Sprite("", {1, 1, 1, 1}, 'line'))
      player:add(Collidable(object.x, object.y, self.tilesWidth, self.tilesHeight, player))
      player:add(Movable("player", "player", 50))
      player:add(Playable())
      self.engine:addEntity(player)
    elseif object.type == "enemy" then
      local enemy = Entity()
      enemy:add(Position(object.x, object.y))
      enemy:add(Size(self.tilesWidth, self.tilesHeight))
      math.randomseed(os.time() + k)
      local r, v, b = math.random(), math.random(), math.random()
      enemy:add(Sprite("", {r, v, b, 1}, 'fill'))
      enemy:add(Collidable(object.x, object.y, self.tilesWidth, self.tilesHeight, enemy))
      enemy:add(Movable("enemy" .. enemyN, "enemy", 50))
      enemy:add(Enemy())
      self.engine:addEntity(enemy)
      table.insert(enemies, enemy)
      enemyN = enemyN+1
    end
  end

  world = bump.newWorld(50)

  world:add(player:get("Collidable"), player:get("Collidable").x,
  player:get("Collidable").y, player:get("Collidable").width,
  player:get("Collidable").height)

  for _, enemy in pairs(enemies) do
    world:add(enemy:get("Collidable"), enemy:get("Collidable").x,
    enemy:get("Collidable").y, enemy:get("Collidable").width,
    enemy:get("Collidable").height)
  end

  map:bump_init(world)

  self.engine:addSystem(DrawSprites(map))
  self.engine:addSystem(DrawMap())
  self.engine:addSystem(DrawHud())
  self.engine:addSystem(MovePlayer())
  self.engine:addSystem(MoveEnemies(map))
  self.engine:addSystem(MoveCharacters(map, world))
end

function Level1:update(dt)
  self.engine:update(dt)
  Lynput.update_(dt)
end

function Level1:draw()
  self.engine:draw()
end
